package com.example.groracleServer;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
/**
 * Class providing database functions
 * @author tdosek
 * @version 1.0
 */
public class CommonDatabaseBroker {
    private String sqlStatement = null;
    private String url = null;
    private String user = null;
    private String password = null;
 // Initializing variables for DB connection
	Connection con = null;
	Statement st = null;
	/**
	 * Constructor of the class
	 */
    public CommonDatabaseBroker(){
    }
    /**
     * Method executing Insert statement
     */
    protected void executeDatabaseStatement(){
    	try {
			st.executeUpdate(sqlStatement);
		} catch (SQLException e) {
			System.out.println("Failed to execute SQL query");
			e.printStackTrace();
		}    		
    }
    /**
     * Method for initialization of database connection
     */
    protected void InitConnection(){
    	// Loading the driver
    	try {
    		Class.forName("org.postgresql.Driver");
    	} catch (ClassNotFoundException e) {
    		System.out.println("Failed to load jdbc driver");
    		e.printStackTrace();
    	}
    	loadDbConnectionInfo();    	
    	try {
    		con = DriverManager.getConnection(url, user, password);
    		st = con.createStatement();
    	}catch (SQLException e){
    		System.out.println("SQL exception occured");
    		e.printStackTrace();
    	}
    }
    /**
     * Method responsible for loading database connection info
     */
    private void loadDbConnectionInfo(){
    	// Reading actual DB configuration
    	ConfigLoader config = new ConfigLoader();
    	config.loadConfig();
    	url = "jdbc:postgresql://" + config.getDbHost() + ":" + config.getDbPort() + "/" + config.getDbInstance();
    	user = config.getDbUserName();
    	password = config.getDbUserPasswd();
    }
    /**
     * Setter method setting SQL query that is to be executed
     * @param query SQL query
     */
    protected void setSqlStatement(String query){
    	sqlStatement = query; 	
    }
    /**
     * Method closing connection to database
     */
    protected void closeDbConnection(){
    	try {
    	if (st != null)
			st.close();
		if (con != null)
			con.close();
    	}catch (SQLException e) {
			System.out.println("Failed to close connections");
			e.printStackTrace();
		}
	}
}
