package com.example.groracleServer;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
/**
 * Class responsible for loading information from configruation file
 * @author tdosek
 * @version 1.0
 */
public class ConfigLoader {
    static final String configPath = "/etc/groracle/config.xml";
    // Setting up to groracle default values, configuration file overrides these though
    private String sockType = "TCP";
    private String sockNum = "9998";
    private String dbHost = "localhost";
    private String dbPort = "5432";
    private String dbInstance = "groracle";
    private String dbUserName = "groracle";
    private String dbUserPasswd = "123456";
    private String grokRulesFile = null;
    private String grokBlacklistFile = null;
    /**
     * Constructor of the class
     */
    public ConfigLoader(){
    }
    /**
     * Method loading information from file and parsing it to variables
     */
    protected void loadConfig(){
   	try {
   	    // Opening the file used for config parsing
   	    File configuration = new File(configPath);
   	    DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
   	    DocumentBuilder docBuilder;

   	    // Building XML document structure
   	    docBuilder = docFactory.newDocumentBuilder();
   	    Document workingConf = docBuilder.parse(configuration);
   	    workingConf.getDocumentElement().normalize();
   	    
   	    // Adding config nodes to node lists
   	    NodeList dbNodeList = workingConf.getElementsByTagName("db");
   	    NodeList listenerNode = workingConf.getElementsByTagName("listener");
   	    NodeList grokNode = workingConf.getElementsByTagName("grok");
   	    
   	    // Setting up the DB config variables with accordance to the config file
   	    for (int iDbNode = 0; iDbNode < dbNodeList.getLength(); iDbNode++){
   		Node nNode = dbNodeList.item(iDbNode);
   		if (nNode.getNodeType() == Node.ELEMENT_NODE) {
   		    NodeList childElements = nNode.getChildNodes();
   		    for (int i = 0; i < childElements.getLength(); i++){
   			if (childElements.item(i).getNodeType() != Node.TEXT_NODE && childElements.item(i).getNodeType() != Node.COMMENT_NODE ){
   			    switch (childElements.item(i).getNodeName()) {
   			    case "postgres-host": this.dbHost = childElements.item(i).getChildNodes().item(0).getTextContent();
   			    case "postgres-port": this.dbPort = childElements.item(i).getChildNodes().item(0).getTextContent();
   			    case "database-name": this.dbInstance = childElements.item(i).getChildNodes().item(0).getTextContent();
   			    case "postgres-user": this.dbUserName = childElements.item(i).getChildNodes().item(0).getTextContent();
   			    case "postgres-password": this.dbUserPasswd = childElements.item(i).getChildNodes().item(0).getTextContent();			
   			    }
   			}
   		    }
   		}
   	    }
   	    // Setting up the listener config variables with accordance to the config file
   	    for (int iListenerNode = 0; iListenerNode < listenerNode.getLength(); iListenerNode++){
   		Node nNode = dbNodeList.item(iListenerNode);
   		if (nNode.getNodeType() == Node.ELEMENT_NODE) {
   		    NodeList childElements = nNode.getChildNodes();
   		    for (int i = 0; i < childElements.getLength(); i++){
   			if (childElements.item(i).getNodeType() != Node.TEXT_NODE && childElements.item(i).getNodeType() != Node.COMMENT_NODE ){
   			    switch (childElements.item(i).getNodeName()) {
   			    case "sock-type": this.sockType = childElements.item(i).getChildNodes().item(0).getTextContent();
   			    case "sockNum": this.sockNum = childElements.item(i).getChildNodes().item(0).getTextContent();	
   			    }
   			}
   		    }
   		}
   	    }
   	    for (int iGrokNode = 0; iGrokNode < grokNode.getLength(); iGrokNode++){
   		Node nNode = grokNode.item(iGrokNode);   		
   		if (nNode.getNodeType() == Node.ELEMENT_NODE) {
   		    NodeList childElements = nNode.getChildNodes();
   		    for (int i = 0; i < childElements.getLength(); i++){
   			if (childElements.item(i).getNodeType() != Node.TEXT_NODE && childElements.item(i).getNodeType() != Node.COMMENT_NODE ){
   			    switch (childElements.item(i).getNodeName()) {
   			    case "pattern-file": this.grokRulesFile = childElements.item(i).getChildNodes().item(0).getTextContent();
   			    case "blacklist-file": this.grokBlacklistFile = childElements.item(i).getChildNodes().item(0).getTextContent();
   			    }
   			}
   		    }
   		}
   	    }
   	} catch (ParserConfigurationException e1) {
   	    System.out.println("Document failed to get parsed");
   	    e1.printStackTrace();
   	} catch (SAXException e) {
   	    System.out.println("Document is not valid XML file");
   	    e.printStackTrace();
   	} catch (IOException e) {
   	    System.out.println("File cannot be accessed");
   	    e.printStackTrace();
   	}	
    }
    // Configuration getters
    /**
     * Method for getting socket type configuration
     * @return socket type (TCP or UX)
     */
    protected String getSockType(){
	return this.sockType;
    }
    /**
     * Method returning socket number (in case of TCP) or socket path (in case of Unix socket)
     * @return socket number or path
     */
    protected String getSockNum(){
	return this.sockNum;
    }
    /**
     * Method returning database host IP
     * @return database host IP
     */
    protected String getDbHost(){
	return this.dbHost;	
    }
    /**
     * Method returning DB port
     * @return database port
     */
    protected String getDbPort(){
	return this.dbPort;
    }
    /**
     * Method returning name of database instance
     * @return database instance name
     */
    protected String getDbInstance(){
	return this.dbInstance;
    }
    /**
     * Method returning database user name
     * @return user name
     */
    protected String getDbUserName(){
	return this.dbUserName;
    }
    /**
     * Method returning database user password
     * @return password
     */
    protected String getDbUserPasswd(){
	return this.dbUserPasswd;
    }
    /**
     * Method returning path to grok rules file path
     * @return grok rule file path
     */
    protected String getGrokRulesFile(){
	return this.grokRulesFile;
    }
    /**
     * Method returning path to blacklist rules file
     * @return path to blacklist rules file
     */
    protected String getGrokBlacklistFile(){
    return this.grokBlacklistFile;
    }
}
