package com.example.groracleServer;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
/**
 * Class providing functionality of communication with discogrok via commandline
 * @author tdosek
 * @version 1.0
 */
public class DiscoGrok {
	private String patternFilePath = null;
	private String unparsedLine = null;
	/**
	 * Constructor of the class
	 * @param unparsedLine Line that is to be examined by discogrok
	 * @param patternFilePath path to the pattern file
	 */
	public DiscoGrok(String unparsedLine, String patternFilePath){
		this.unparsedLine = unparsedLine;
		this.patternFilePath = patternFilePath;		
	}
	/**
	 * Method providing results of discovery of a pattern suggested by grok
	 * @return suggestion from discogrok
	 */
	protected String suggestPattern(){
		// Composing commandline of the discogrok call
		String cmd = "echo " + unparsedLine + " | discogrok --patterns " + patternFilePath;
		// Allocating return pattern variable
		String pattern = null;
		try {
	    		// Creating process for execution of the command
			Process proc = Runtime.getRuntime().exec(new String[] {"/usr/bin/bash", "-c", cmd});
			try {
				//  Waiting until the pattern is generated by discogrok
				proc.waitFor();
			} catch (InterruptedException e) {
				System.out.println("Commandline call was interrupted!");
				e.printStackTrace();
			}
			// Reading the commandline call return
			BufferedReader readCmdOut = new BufferedReader(new InputStreamReader(proc.getInputStream()));
			while (readCmdOut.ready()){
				pattern = readCmdOut.readLine();
			}
		} catch (IOException e) {
			System.out.println("Reaching discogrok failed. Please revise your configuration "
					+ "file or discogrok installation");
			e.printStackTrace();
		}		
		return pattern;
	}
}
